//-------+---------+---------+---------+---------+---------+---------+--------=
//
// File: diag_hash.h
//
//----------

#ifndef diag_hash_H				// (prevent multiple inclusion)
#define diag_hash_H

// other files

#include "utilities.h"			// utility stuff
#include "sequences.h"			// sequence stuff

// establish ownership of global variables

#ifdef diag_hash_owner
#define global
#else
#define global extern
#endif

//----------
//
// diagonal hash--
//	(see diag_hash.c for details)
//
// Helpful formulae:
//	diag = pos1 - pos2
//	pos1 = diag + pos2
//	pos2 = pos1 - diag
//
//----------

// note that diagHashSize must be a power of two

#ifdef tiny_diag_hash			// test case; smaller hash forces collisions
#define diagHashSize			((u32) 1024)
#else							// normal case
#ifdef huge_diag_hash			// test case; no collisions on 1M vs 1M
#define diagHashSize			((u32) 4194304)
#else							// normal case
#define diagHashSize			((u32) 65536)
#endif
#endif

#define diagNumber(pos1,pos2)	(((sgnpos)(pos1))-((sgnpos)(pos2)))
#define hashedDiag(pos1,pos2)	((diagNumber(pos1,pos2)) & (diagHashSize-1))

//--- initialized variables for this module ---

#ifdef diag_hash_owner
unspos*  diagEnd    = NULL;		// arrays (indexed by 0..diagHashSize-1) to
unspos*  diagStart  = NULL;		// .. track the extent of discovered hits on a
sgnpos*  diagActual = NULL;		// .. given diagonal (modulo diagHashSize);
								// .. no values in any of these arrays are valid
								// .. unless the corresponding value in
								// .. diagStart is non-zero.  diagStart,diagEnd
								// .. records the most recent hit, as positions
								// .. in sequence 2;  diagActual records the
								// .. actual diagonal represented (this is used
								// .. only to detect collisions)

int*     diagActive = NULL;		// array (indexed by 0..numdiagActive-1) of
int      numDiagActive;			// .. positions in diagEnd and diagStart that
								// ..  have a non-zero value

//--- external access to the variables, for other modules ---

#else
extern unspos* diagEnd;
extern unspos* diagStart;
extern sgnpos* diagActual;
extern int*    diagActive;
extern int     numDiagActive;
#endif

// macros for accessing the hash

#define activate_hashed_diag(h) diagActive[numDiagActive++] = h;

//----------
//
// prototypes for routines in diag_hash.c
//
//----------

void empty_diag_hash (void);
void free_diag_hash  (void);

#undef global
#endif // diag_hash_H
